@@ -3,5 +3,5 @@ from .metaApi.historyStorage import HistoryStorage
 from .metaApi.memoryHistoryStorage import MemoryHistoryStorage
 from .clients.metaApi.synchronizationListener import SynchronizationListener
 from .metaApi.models import format_error, format_date, date
-from metaapi_cloud_copyfactory_sdk import CopyFactory
+from metaapi_cloud_copyfactory_sdk import CopyFactory, StopoutListener
 from metaapi_cloud_metastats_sdk import MetaStats
