@@ -119,15 +119,6 @@ class MetatraderAccountModel(ABC):
             Flag indicating if trades should be placed as manual trades on this account.
         """
 
-    @property
-    @abstractmethod
-    def extensions(self):
-        """Returns API extensions.
-
-        Returns:
-            API extensions.
-        """
-
     @property
     @abstractmethod
     def metadata(self) -> Dict:
@@ -196,6 +187,15 @@ class MetatraderAccountModel(ABC):
             Account version value.
         """
 
+    @property
+    @abstractmethod
+    def region(self) -> str:
+        """Returns account region.
+
+        Returns:
+            Account region value.
+        """
+
     @abstractmethod
     async def reload(self):
         """Reloads MetaTrader account from API.
