@@ -8,6 +8,7 @@ import pytz
 import re
 import traceback
 import json
+import asyncio
 
 
 def date(date_time: str or float or int) -> datetime:
@@ -83,6 +84,21 @@ def string_format_error(err: Exception or any):
     return json.dumps(format_error(err))
 
 
+async def promise_any(coroutines):
+    exception_task = None
+    while len(coroutines):
+        done, coroutines = await asyncio.wait(coroutines, return_when=asyncio.FIRST_COMPLETED)
+        for task in done:
+            if not exception_task and task.exception():
+                exception_task = task
+            else:
+                for wait_task in coroutines:
+                    wait_task.cancel()
+                return task.result()
+
+    return exception_task.result()
+
+
 class G1Encoder(json.JSONEncoder):
     """A JSON encoder used to encode cloud-g1 account terminal data."""
     def iterencode(self, obj, _one_shot=False):
@@ -202,6 +218,53 @@ class MetatraderAccountInformation(TypedDict, total=False):
     """Account credit in the deposit currency."""
 
 
+class StopLossThreshold(TypedDict):
+    """Stop loss threshold."""
+
+    threshold: float
+    """Price threshold relative to position open price, interpreted according to units field value."""
+    stopLoss: float
+    """Stop loss value, interpreted according to units and basePrice field values."""
+
+
+class ThresholdTrailingStopLoss(TypedDict, total=False):
+    """Threshold trailing stop loss configuration."""
+
+    thresholds: List[StopLossThreshold]
+    """Stop loss thresholds."""
+    units: Optional[str]
+    """Threshold stop loss units. ABSOLUTE_PRICE means the that the value of stop loss threshold fields contain a
+    final threshold & stop loss value. RELATIVE* means that the threshold fields value contains relative threshold &
+    stop loss values, expressed either in price, points, pips, account currency or balance percentage. Default is
+    ABSOLUTE_PRICE. One of ABSOLUTE_PRICE, RELATIVE_PRICE, RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY,
+    RELATIVE_BALANCE_PERCENTAGE."""
+    stopPriceBase: Optional[str]
+    """Defines the base price to calculate SL relative to for POSITION_MODIFY and pending order requests. Default is
+    OPEN_PRICE. One of CURRENT_PRICE, OPEN_PRICE."""
+
+
+class DistanceTrailingStopLoss(TypedDict, total=False):
+    """Distance trailing stop loss configuration."""
+
+    distance: Optional[float]
+    """SL distance relative to current price, interpreted according to units field value."""
+    units: Optional[str]
+    """Distance trailing stop loss units. RELATIVE_* means that the distance field value contains relative stop loss
+    expressed either in price, points, pips, account currency or balance percentage. Default is RELATIVE_PRICE. One of
+    RELATIVE_PRICE, RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE"""
+
+
+class TrailingStopLoss(TypedDict, total=False):
+    """Distance trailing stop loss configuration."""
+
+    distance: Optional[DistanceTrailingStopLoss]
+    """Distance trailing stop loss configuration. If both distance and threshold TSL are set, then the resulting SL
+    will be the one which is closest to the current price."""
+    threshold: Optional[ThresholdTrailingStopLoss]
+    """Threshold trailing stop loss configuration. If both distance and threshold TSL are set, then the resulting SL
+    will be the one which is closest to the current price."""
+
+
 class MetatraderPosition(TypedDict, total=False):
     """MetaTrader position"""
 
@@ -229,12 +292,31 @@ class MetatraderPosition(TypedDict, total=False):
     """Optional position stop loss price."""
     takeProfit: Optional[float]
     """Optional position take profit price."""
+    trailingStopLoss: Optional[TrailingStopLoss]
+    """Distance trailing stop loss configuration."""
     volume: float
     """Position volume."""
-    swap: float
-    """Position cumulative swap."""
     profit: float
-    """Position cumulative profit."""
+    """Position cumulative profit, including unrealized profit resulting from currently open position part (except
+    swap and commissions) and realized profit resulting from partially closed position part and including swap and
+    commissions."""
+    realizedProfit: float
+    """Profit of the already closed part, including commissions and swap (realized and unrealized)."""
+    unrealizedProfit: float
+    """Profit of the part of the position which is not yet closed, excluding swap and commissions."""
+    swap: float
+    """Position cumulative swap, including both swap from currently open position part (unrealized
+    swap) and swap from partially closed position part (realized swap)."""
+    realizedSwap: float
+    """Swap from partially closed position part."""
+    unrealizedSwap: float
+    """Swap resulting from currently open position part."""
+    commission: float
+    """Total position commissions, resulting both from currently open and closed position parts."""
+    realizedCommission: float
+    """Position realized commission, resulting from partially closed position part."""
+    unrealizedCommission: float
+    """Position unrealized commission, resulting from currently open position part."""
     comment: Optional[str]
     """Optional position comment. The sum of the line lengths of the comment and the clientId
     must be less than or equal to 26. For more information see https://metaapi.cloud/docs/client/clientIdUsage/"""
@@ -244,12 +326,6 @@ class MetatraderPosition(TypedDict, total=False):
     your trades to objects in your application and then track trade progress. The sum of the line lengths of the
     comment and the clientId must be less than or equal to 26. For more information see
     https://metaapi.cloud/docs/client/clientIdUsage/"""
-    unrealizedProfit: float
-    """Profit of the part of the position which is not yet closed, including swap."""
-    realizedProfit: float
-    """Profit of the already closed part, including commissions and swap."""
-    commission: Optional[float]
-    """Optional position commission."""
     reason: str
     """Position opening reason. One of POSITION_REASON_CLIENT, POSITION_REASON_EXPERT, POSITION_REASON_MOBILE,
     POSITION_REASON_WEB, POSITION_REASON_UNKNOWN. See
@@ -289,12 +365,16 @@ class MetatraderOrder(TypedDict, total=False):
     """Order symbol."""
     openPrice: float
     """Order open price (market price for market orders, limit price for limit orders or stop price for stop orders)."""
+    stopLimitPrice: Optional[float]
+    """The limit order price for the StopLimit order."""
     currentPrice: Optional[float]
     """Current price, filled for pending orders only. Not filled for history orders."""
     stopLoss: Optional[float]
     """Optional order stop loss price."""
     takeProfit: Optional[float]
     """Optional order take profit price."""
+    trailingStopLoss: Optional[TrailingStopLoss]
+    """Distance trailing stop loss configuration."""
     volume: float
     """Order requested quantity."""
     currentVolume: float
@@ -330,8 +410,6 @@ class MetatraderOrder(TypedDict, total=False):
     """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
     closeByPositionId: Optional[str]
     """Identifier of an opposite position used for closing by order ORDER_TYPE_CLOSE_BY"""
-    stopLimitPrice: Optional[float]
-    """The limit order price for the StopLimit order."""
 
 
 class MetatraderHistoryOrders(TypedDict):
@@ -400,6 +478,12 @@ class MetatraderDeal(TypedDict, total=False):
     https://www.mql5.com/en/docs/constants/tradingconstants/dealproperties#enum_deal_reason."""
     accountCurrencyExchangeRate: Optional[float]
     """Current exchange rate of account currency into account base currency (USD if you did not override it)."""
+    stopLoss: Optional[float]
+    """Deal stop loss. For MT5 opening deal this is the SL of the order opening the position. For MT4 deals or MT5
+    closing deal this is the last known position SL."""
+    takeProfit: Optional[float]
+    """Deal take profit. For MT5 opening deal this is the TP of the order opening the position. For MT4 deals or MT5
+    closing deal this is the last known position TP."""
 
 
 class MetatraderDeals(TypedDict):
@@ -555,6 +639,12 @@ class MetatraderSymbolSpecification(TypedDict, total=False):
     """Date of the symbol trade beginning (usually used for futures)."""
     expirationTime: Optional[datetime]
     """Date of the symbol trade end (usually used for futures)."""
+    pipSize: Optional[float]
+    """Size of a pip. Pip size is defined for spot and CFD symbols only."""
+    stopsLevel: float
+    """Minimal indention in points from the current close price to place stop orders."""
+    freezeLevel: float
+    """Distance to freeze trade operations in points."""
 
 
 class MetatraderSymbolPrice(TypedDict):
@@ -619,7 +709,7 @@ class TradeOptions(TypedDict, total=False):
     symbol specification. Not used for close by orders."""
 
 
-class MarketTradeOptions(TradeOptions):
+class MarketTradeOptions(TradeOptions, total=False):
     """Market trade options."""
 
     fillingModes: Optional[List[str]]
@@ -629,6 +719,16 @@ class MarketTradeOptions(TradeOptions):
     explanation."""
 
 
+class CreateMarketTradeOptions(MarketTradeOptions, total=False):
+    """Create market trade options."""
+
+    trailingStopLoss: Optional[TrailingStopLoss]
+    """Distance trailing stop loss configuration."""
+    stopPriceBase: Optional[str]
+    """Defines the base price to calculate SL/TP relative to for pending order requests. Default is CURRENT_PRICE,
+    one of CURRENT_PRICE."""
+
+
 class ExpirationOptions(TypedDict, total=False):
     """Pending order expiration settings."""
 
@@ -647,6 +747,58 @@ class PendingTradeOptions(TradeOptions):
 
     expiration: Optional[ExpirationOptions]
     """Optional pending order expiration settings. See Pending order expiration settings section."""
+    trailingStopLoss: Optional[TrailingStopLoss]
+    """Distance trailing stop loss configuration."""
+    stopPriceBase: Optional[str]
+    """Defines the base price to calculate SL/TP relative to for *_MODIFY and pending order requests. STOP_PRICE means
+    the SL/TP is relative to previous SL/TP value. Default is OPEN_PRICE, one of CURRENT_PRICE, OPEN_PRICE."""
+    openPriceUnits: Optional[str]
+    """Open price units. ABSOLUTE_PRICE means the that the value of openPrice field is a final open price value.
+    RELATIVE* means that the openPrice field value contains relative open price expressed either in price, points,
+    pips, account currency or balance percentage. Default is ABSOLUTE_PRICE. One of ABSOLUTE_PRICE, RELATIVE_PRICE,
+    RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+
+
+class StopLimitPendingTradeOptions(PendingTradeOptions, total=False):
+    """Options for creating a stop limit pending order."""
+
+    openPriceBase: Optional[str]
+    """Defines the base price to calculate open price relative to for ORDER_MODIFY and pending order requests. Default
+    is CURRENT_PRICE for pending orders or STOP_LIMIT_PRICE for stop limit orders. One of CURRENT_PRICE, OPEN_PRICE,
+    STOP_LIMIT_PRICE."""
+    stopLimitPriceUnits: Optional[str]
+    """Stop limit price units. ABSOLUTE_PRICE means the that the value of stopLimitPrice field is a final stop limit
+    price value. RELATIVE* means that the stopLimitPrice field value contains relative stop limit price expressed
+    either in price, points, pips, account currency or balance percentage. Default is ABSOLUTE_PRICE. One of
+    ABSOLUTE_PRICE, RELATIVE_PRICE, RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+
+
+class ModifyOrderOptions(TypedDict, total=False):
+    """Options for modifying orders."""
+
+    trailingStopLoss: Optional[TrailingStopLoss]
+    """Distance trailing stop loss configuration."""
+    stopPriceBase: Optional[str]
+    """Defines the base price to calculate SL/TP relative to for *_MODIFY and pending order requests. STOP_PRICE means
+    the SL/TP is relative to previous SL/TP value. Default is OPEN_PRICE, one of CURRENT_PRICE, OPEN_PRICE,
+    STOP_PRICE."""
+    openPriceUnits: Optional[str]
+    """Open price units. ABSOLUTE_PRICE means the that the value of openPrice field is a final open price value.
+    RELATIVE* means that the openPrice field value contains relative open price expressed either in price, points,
+    pips, account currency or balance percentage. Default is ABSOLUTE_PRICE. One of ABSOLUTE_PRICE, RELATIVE_PRICE,
+    RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+    openPriceBase: Optional[str]
+    """Defines the base price to calculate open price relative to for ORDER_MODIFY and pending order requests. Default
+    is CURRENT_PRICE for pending orders or STOP_LIMIT_PRICE for stop limit orders. One of CURRENT_PRICE, OPEN_PRICE,
+    STOP_LIMIT_PRICE."""
+    stopLimitPriceUnits: Optional[str]
+    """Stop limit price units. ABSOLUTE_PRICE means the that the value of stopLimitPrice field is a final stop limit
+    price value. RELATIVE* means that the stopLimitPrice field value contains relative stop limit price expressed
+    either in price, points, pips, account currency or balance percentage. Default is ABSOLUTE_PRICE. One of
+    ABSOLUTE_PRICE, RELATIVE_PRICE, RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+    stopLimitPriceBase: Optional[str]
+    """Defines the base price to calculate stop limit price relative to for ORDER_MODIFY requests. One of
+    CURRENT_PRICE, STOP_LIMIT_PRICE."""
 
 
 class ValidationDetails(TypedDict, total=False):
@@ -768,7 +920,27 @@ class StopOptions(TypedDict):
     value: float
     """Stop (SL or TP) value."""
     units: str
-    """stop units. ABSOLUTE_PRICE means the that the value of value field is a final stop value.
-    RELATIVE_* means that the value field value contains relative stop expressed either in price, points, account
+    """Stop units. ABSOLUTE_PRICE means the that the value of value field is a final stop value.
+    RELATIVE_* means that the value field value contains relative stop expressed either in price, points, pips, account
     currency or balance percentage. Default is ABSOLUTE_PRICE. Allowed values are ABSOLUTE_PRICE, RELATIVE_PRICE,
-    RELATIVE_POINTS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+    RELATIVE_POINTS, RELATIVE_PIPS, RELATIVE_CURRENCY, RELATIVE_BALANCE_PERCENTAGE."""
+
+
+class ServerTime(TypedDict):
+    """Current server time (see https://metaapi.cloud/docs/client/models/serverTime/)."""
+    time: datetime
+    """Current server time."""
+    brokerTime: str
+    """Current broker time, in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
+    lastQuoteTime: Optional[datetime]
+    """Last quote time."""
+    lastQuoteBrokerTime: Optional[str]
+    """Last quote time, in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
+
+
+class QuoteTime(TypedDict):
+    """Quote time."""
+    time: datetime
+    """Quote time."""
+    brokerTime: str
+    """Quote time in broker timezone, YYYY-MM-DD HH:mm:ss.SSS format."""
